clear
addpath(genpath('../cellLib/Scripts/'))
file='Vars.csv';
% build new components from the .csv
comp_dir=pwd;
[comp,idx]=newComp([comp_dir filesep file]);

% copy the existing components if need
% read the existing model and get the existing components
%comp=copyComp(comp,listComp,idx);

% Encapsuate new components
encap=["Cai"];
chdname=["JVOCCi","JNCXi","JSERCAi","JCICRi","JPMCAi","Jleaki"];
labels=cell(length(chdname),1);
chd={};
for i=1:length(chdname)
    chd(i).name= chdname(i);
    chd(i).def= chdname(i);
    chd(i).label=labels{i,1};
end
comp=encapNew(comp,encap,chd,idx);

encap=["Casr"];
chdname=["JSERCAi","JCICRi","Jleaki"];
labels=cell(length(chdname),1);
chd={};
for i=1:length(chdname)
    chd(i).name= chdname(i);
    chd(i).def= chdname(i);
    chd(i).label=labels{i,1};
end
comp=encapNew(comp,encap,chd,idx);

encap=["Vm"];
chdname=["JNKA","JCli","JVOCCi","JNCXi","JKi"];
labels=cell(length(chdname),1);
chd={};
for i=1:length(chdname)
    chd(i).name= chdname(i);
    chd(i).def= chdname(i);
    chd(i).label=labels{i,1};
end
comp=encapNew(comp,encap,chd,idx);

comp=getPara(comp,idx);
comp=updateVIO(comp,idx);
% Add equations to components
Eqs=["J_VOCCi=G_Ca*((v_Ca1-v_i)/(1{dimensionless}+exp((v_Ca2-v_i)/R_Ca)));",...
    "J_NCXi=G_NCX*(c_i/(c_i+c_NCX))*(v_NCX-v_i);",...
    "J_SERCAi=B*(pow(c_i,2{dimensionless})/(pow(c_i,2{dimensionless})+pow(c_b,2{dimensionless})));",...
    "J_CICRi=C*(pow(s_i,2{dimensionless})/(pow(s_i,2{dimensionless})+pow(s_c,2{dimensionless})))*(pow(c_i,4{dimensionless})/(pow(c_i,4{dimensionless})+pow(c_c,4{dimensionless})));",...
    "J_PMCAi=D*c_i*(1{dimensionless}+(v_i-v_d)/(R_d));",...
    "J_leaki=L*s_i;",...
    "J_NKA=C_NKA*(K_out/(K_mK+K_out));",...
    "J_Cli=G_Cl*(v_i-v_Cl);",...
    "J_Ki=G_Ki*w_i*(v_i-v_K);",...
    "ode(w_i,t)=lambda*(K_activationi-w_i);",...
    "ode(c_i,t)=J_VOCCi-J_NCXi-J_SERCAi+J_CICRi-J_PMCAi+J_leaki;",...
    "ode(s_i,t)=J_SERCAi-J_CICRi-J_leaki;",...
    "ode(v_i,t)=gamma*(-J_NKAi-J_Cli+2{dimensionless}*J_VOCCi+J_NCXi-J_Ki);"];

for i=1:8
    comp(i).Eqs=Eqs(i);
end
comp(9).Eqs=Eqs(9:10);
for i=10:12
    comp(i).Eqs=Eqs(i+1);
end

% Build models
cmnames=string(extractfield(comp,'name'));
modelnames=cmnames([1:9,13]);
listComp={};
labels=cell(length(modelnames),1);

for i=1:length(modelnames)
    modelname=modelnames(i);
    listComp.name=modelname;
    listComp.def=modelname;
    listComp.from="";
    listComp.impComp="";
    listComp.defaultVars="Y";
    listComp.label=labels{i,1};
    listMap=strings(0,2);
    model=buildModel(modelname,comp,listComp,listMap,idx);
    modelfile=strcat(modelname,'.txt');
    writeModel(model,comp,modelfile,idx)
end

modelnames=cmnames(10:12);
for n=1:length(modelnames)
modelname=modelnames(n);
icomp=comp(cmnames==modelname);
chd=icomp.children;
listComp={};
labels=cell(length(chd)+1,1);
listMap=strings(0,2);
for i=1:length(chd)
    ichd=chd(i);
    listComp(i).name=ichd;
    listComp(i).def=ichd;
    listComp(i).from=strcat(ichd,".cellml");
    listComp(i).impComp=ichd;
    listComp(i).defaultVars="Y";
    listComp(i).label=labels{i,1};
    listMap(i,:)=[modelname,ichd];
end
i=i+1;
listComp(i).name=modelname;
listComp(i).def=modelname;
listComp(i).from="";
listComp(i).impComp="";
listComp(i).defaultVars="Y";
listComp(i).label=labels{i,1};

model=buildModel(modelname,comp,listComp,listMap,idx);
modelfile=strcat(modelname,'.txt');
writeModel(model,comp,modelfile,idx)
end